@@ -9,12 +9,10 @@ import (
 	"flag"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"os/exec"
 	"path/filepath"
-	"regexp"
 	"runtime"
 	"strings"
 )
@@ -26,19 +24,6 @@ var ForbiddenImports = map[string]bool{
 }
 
 var runCrossCompile = flag.Bool("cross-compile", true, "run cross compilation tests")
-var minioServer = flag.String("minio", "", "path to the minio server binary")
-var debug = flag.Bool("debug", false, "output debug messages")
-
-var minioServerEnv = map[string]string{
-	"MINIO_ACCESS_KEY": "KEBIYDZ87HCIH5D17YCN",
-	"MINIO_SECRET_KEY": "bVX1KhipSBPopEfmhc7rGz8ooxx27xdJ7Gkh1mVe",
-}
-
-var minioEnv = map[string]string{
-	"RESTIC_TEST_S3_SERVER": "http://127.0.0.1:9000",
-	"AWS_ACCESS_KEY_ID":     "KEBIYDZ87HCIH5D17YCN",
-	"AWS_SECRET_ACCESS_KEY": "bVX1KhipSBPopEfmhc7rGz8ooxx27xdJ7Gkh1mVe",
-}
 
 func init() {
 	flag.Parse()
@@ -54,22 +39,11 @@ type CIEnvironment interface {
 // TravisEnvironment is the environment in which Travis tests run.
 type TravisEnvironment struct {
 	goxOSArch []string
-	minio     string
-
-	minioSrv     *Background
-	minioTempdir string
-
-	env map[string]string
+	env       map[string]string
 }
 
 func (env *TravisEnvironment) getMinio() error {
-	if *minioServer != "" {
-		msg("using minio server at %q\n", *minioServer)
-		env.minio = *minioServer
-		return nil
-	}
-
-	tempfile, err := ioutil.TempFile("", "minio-server-")
+	tempfile, err := os.Create(filepath.Join(os.Getenv("GOPATH"), "bin", "minio"))
 	if err != nil {
 		return fmt.Errorf("create tempfile for minio download failed: %v\n", err)
 	}
@@ -104,41 +78,6 @@ func (env *TravisEnvironment) getMinio() error {
 	}
 
 	msg("downloaded minio server to %v\n", tempfile.Name())
-	env.minio = tempfile.Name()
-	return nil
-}
-
-func (env *TravisEnvironment) runMinio() error {
-	if env.minio == "" {
-		return nil
-	}
-
-	// start minio server
-	msg("starting minio server at %s", env.minio)
-
-	dir, err := ioutil.TempDir("", "minio-root")
-	if err != nil {
-		return fmt.Errorf("running minio server failed: %v", err)
-	}
-
-	env.minioSrv, err = StartBackgroundCommand(minioServerEnv, env.minio,
-		"server",
-		"--address", "127.0.0.1:9000",
-		dir)
-	if err != nil {
-		return fmt.Errorf("error running minio server: %v", err)
-	}
-
-	// go func() {
-	// 	time.Sleep(300 * time.Millisecond)
-	// 	env.minioSrv.Cmd.Process.Kill()
-	// }()
-
-	for k, v := range minioEnv {
-		env.env[k] = v
-	}
-
-	env.minioTempdir = dir
 	return nil
 }
 
@@ -148,10 +87,14 @@ func (env *TravisEnvironment) Prepare() error {
 
 	msg("preparing environment for Travis CI\n")
 
-	for _, pkg := range []string{
+	pkgs := []string{
 		"golang.org/x/tools/cmd/cover",
 		"github.com/pierrre/gotestcover",
-	} {
+		"github.com/NebulousLabs/glyphcheck",
+		"github.com/restic/rest-server/cmd/rest-server",
+	}
+
+	for _, pkg := range pkgs {
 		err := run("go", "get", pkg)
 		if err != nil {
 			return err
@@ -161,11 +104,8 @@ func (env *TravisEnvironment) Prepare() error {
 	if err := env.getMinio(); err != nil {
 		return err
 	}
-	if err := env.runMinio(); err != nil {
-		return err
-	}
 
-	if *runCrossCompile && !(runtime.Version() < "go1.7") {
+	if *runCrossCompile {
 		// only test cross compilation on linux with Travis
 		if err := run("go", "get", "github.com/mitchellh/gox"); err != nil {
 			return err
@@ -177,25 +117,13 @@ func (env *TravisEnvironment) Prepare() error {
 				"darwin/386", "darwin/amd64",
 				"freebsd/386", "freebsd/amd64",
 				"openbsd/386", "openbsd/amd64",
-			}
-			if !strings.HasPrefix(runtime.Version(), "go1.3") {
-				env.goxOSArch = append(env.goxOSArch,
-					"linux/arm", "freebsd/arm")
+				"linux/arm", "freebsd/arm",
 			}
 		} else {
 			env.goxOSArch = []string{runtime.GOOS + "/" + runtime.GOARCH}
 		}
 
 		msg("gox: OS/ARCH %v\n", env.goxOSArch)
-
-		if runtime.Version() < "go1.5" {
-			err := run("gox", "-build-toolchain",
-				"-osarch", strings.Join(env.goxOSArch, " "))
-
-			if err != nil {
-				return err
-			}
-		}
 	}
 
 	return nil
@@ -204,107 +132,15 @@ func (env *TravisEnvironment) Prepare() error {
 // Teardown stops backend services and cleans the environment again.
 func (env *TravisEnvironment) Teardown() error {
 	msg("run travis teardown\n")
-	if env.minioSrv != nil {
-		msg("stopping minio server\n")
-
-		if env.minioSrv.Cmd.ProcessState == nil {
-			err := env.minioSrv.Cmd.Process.Kill()
-			if err != nil {
-				fmt.Fprintf(os.Stderr, "error killing minio server process: %v", err)
-			}
-		} else {
-			result := <-env.minioSrv.Result
-			if result.Error != nil {
-				msg("minio server returned error: %v\n", result.Error)
-				msg("stdout: %s\n", result.Stdout)
-				msg("stderr: %s\n", result.Stderr)
-			}
-		}
-
-		err := os.RemoveAll(env.minioTempdir)
-		if err != nil {
-			msg("error removing minio tempdir %v: %v\n", env.minioTempdir, err)
-		}
-	}
-
 	return nil
 }
 
-func goVersionAtLeast151() bool {
-	v := runtime.Version()
-
-	if match, _ := regexp.MatchString(`^go1\.[0-4]`, v); match {
-		return false
-	}
-
-	if v == "go1.5" {
-		return false
-	}
-
-	return true
-}
-
-// Background is a program running in the background.
-type Background struct {
-	Cmd    *exec.Cmd
-	Result chan Result
-}
-
-// Result is the result of a program that ran in the background.
-type Result struct {
-	Stdout, Stderr string
-	Error          error
-}
-
-// StartBackgroundCommand runs a program in the background.
-func StartBackgroundCommand(env map[string]string, cmd string, args ...string) (*Background, error) {
-	msg("running background command %v %v\n", cmd, args)
-	b := Background{
-		Result: make(chan Result, 1),
-	}
-
-	stdout := bytes.NewBuffer(nil)
-	stderr := bytes.NewBuffer(nil)
-
-	c := exec.Command(cmd, args...)
-	c.Stdout = stdout
-	c.Stderr = stderr
-
-	if *debug {
-		c.Stdout = io.MultiWriter(c.Stdout, os.Stdout)
-		c.Stderr = io.MultiWriter(c.Stderr, os.Stderr)
-	}
-	c.Env = updateEnv(os.Environ(), env)
-
-	b.Cmd = c
-
-	err := c.Start()
-	if err != nil {
-		msg("error starting background job %v: %v\n", cmd, err)
-		return nil, err
-	}
-
-	go func() {
-		err := b.Cmd.Wait()
-		msg("background job %v returned: %v\n", cmd, err)
-		msg("stdout: %s\n", stdout.Bytes())
-		msg("stderr: %s\n", stderr.Bytes())
-		b.Result <- Result{
-			Stdout: string(stdout.Bytes()),
-			Stderr: string(stderr.Bytes()),
-			Error:  err,
-		}
-	}()
-
-	return &b, nil
-}
-
 // RunTests starts the tests for Travis.
 func (env *TravisEnvironment) RunTests() error {
 	// do not run fuse tests on darwin
 	if runtime.GOOS == "darwin" {
 		msg("skip fuse integration tests on %v\n", runtime.GOOS)
-		os.Setenv("RESTIC_TEST_FUSE", "0")
+		_ = os.Setenv("RESTIC_TEST_FUSE", "0")
 	}
 
 	cwd, err := os.Getwd()
@@ -314,7 +150,37 @@ func (env *TravisEnvironment) RunTests() error {
 
 	env.env["GOPATH"] = cwd + ":" + filepath.Join(cwd, "vendor")
 
-	if *runCrossCompile && !(runtime.Version() < "go1.7") {
+	// ensure that the following tests cannot be silently skipped on Travis
+	ensureTests := []string{
+		"restic/backend/rest.TestBackendREST",
+		"restic/backend/sftp.TestBackendSFTP",
+		"restic/backend/s3.TestBackendMinio",
+	}
+
+	// if the test s3 repository is available, make sure that the test is not skipped
+	if os.Getenv("RESTIC_TEST_S3_REPOSITORY") != "" {
+		ensureTests = append(ensureTests, "restic/backend/s3.TestBackendS3")
+	} else {
+		msg("S3 repository not available\n")
+	}
+
+	// if the test swift service is available, make sure that the test is not skipped
+	if os.Getenv("RESTIC_TEST_SWIFT") != "" {
+		ensureTests = append(ensureTests, "restic/backend/swift.TestBackendSwift")
+	} else {
+		msg("Swift service not available\n")
+	}
+
+	// if the test b2 repository is available, make sure that the test is not skipped
+	if os.Getenv("RESTIC_TEST_B2_REPOSITORY") != "" {
+		ensureTests = append(ensureTests, "restic/backend/b2.TestBackendB2")
+	} else {
+		msg("B2 repository not available\n")
+	}
+
+	env.env["RESTIC_TEST_DISALLOW_SKIP"] = strings.Join(ensureTests, ",")
+
+	if *runCrossCompile {
 		// compile for all target architectures with tags
 		for _, tags := range []string{"release", "debug"} {
 			err := runWithEnv(env.env, "gox", "-verbose",
@@ -343,6 +209,10 @@ func (env *TravisEnvironment) RunTests() error {
 		return err
 	}
 
+	if err = runGlyphcheck(); err != nil {
+		return err
+	}
+
 	deps, err := findImports()
 	if err != nil {
 		return err
@@ -387,16 +257,16 @@ func (env *AppveyorEnvironment) Teardown() error {
 // findGoFiles returns a list of go source code file names below dir.
 func findGoFiles(dir string) (list []string, err error) {
 	err = filepath.Walk(dir, func(name string, fi os.FileInfo, err error) error {
-		if filepath.Base(name) == "vendor" {
+		relpath, err := filepath.Rel(dir, name)
+		if err != nil {
+			return err
+		}
+
+		if relpath == "vendor" || relpath == "pkg" {
 			return filepath.SkipDir
 		}
 
-		if filepath.Ext(name) == ".go" {
-			relpath, err := filepath.Rel(dir, name)
-			if err != nil {
-				return err
-			}
-
+		if filepath.Ext(relpath) == ".go" {
 			list = append(list, relpath)
 		}
 
@@ -498,6 +368,18 @@ func runGofmt() error {
 	return nil
 }
 
+func runGlyphcheck() error {
+	cmd := exec.Command("glyphcheck", "./...")
+	cmd.Stderr = os.Stderr
+
+	buf, err := cmd.Output()
+	if err != nil {
+		return fmt.Errorf("error running glyphcheck: %v\noutput: %s\n", err, buf)
+	}
+
+	return nil
+}
+
 func run(command string, args ...string) error {
 	msg("run %v %v\n", command, strings.Join(args, " "))
 	return runWithEnv(nil, command, args...)
