@@ -1,10 +1,15 @@
 package main
 
 import (
+	"bufio"
+	"bytes"
 	"fmt"
+	"log"
 	"os"
 	"restic"
 	"restic/debug"
+	"restic/options"
+	"runtime"
 
 	"github.com/spf13/cobra"
 
@@ -22,18 +27,39 @@ directories in an encrypted repository stored on different backends.
 	SilenceErrors: true,
 	SilenceUsage:  true,
 
-	// run the debug functions for all subcommands (if build tag "debug" is
-	// enabled)
 	PersistentPreRunE: func(*cobra.Command, []string) error {
-		return runDebug()
+		// parse extended options
+		opts, err := options.Parse(globalOptions.Options)
+		if err != nil {
+			return err
+		}
+		globalOptions.extended = opts
+
+		// run the debug functions for all subcommands (if build tag "debug" is
+		// enabled)
+		if err := runDebug(); err != nil {
+			return err
+		}
+
+		return nil
 	},
 	PersistentPostRun: func(*cobra.Command, []string) {
 		shutdownDebug()
 	},
 }
 
+var logBuffer = bytes.NewBuffer(nil)
+
+func init() {
+	// install custom global logger into a buffer, if an error occurs
+	// we can show the logs
+	log.SetOutput(logBuffer)
+}
+
 func main() {
 	debug.Log("main %#v", os.Args)
+	debug.Log("restic %s, compiled with %v on %v/%v",
+		version, runtime.Version(), runtime.GOOS, runtime.GOARCH)
 	err := cmdRoot.Execute()
 
 	switch {
@@ -43,6 +69,14 @@ func main() {
 		fmt.Fprintf(os.Stderr, "%v\n", err)
 	case err != nil:
 		fmt.Fprintf(os.Stderr, "%+v\n", err)
+
+		if logBuffer.Len() > 0 {
+			fmt.Fprintf(os.Stderr, "also, the following messages were logged by a library:\n")
+			sc := bufio.NewScanner(logBuffer)
+			for sc.Scan() {
+				fmt.Fprintln(os.Stderr, sc.Text())
+			}
+		}
 	}
 
 	var exitCode int
