@@ -2,6 +2,7 @@ package test
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -13,133 +14,56 @@ import (
 	"sort"
 	"strings"
 	"testing"
+	"time"
 
 	"restic/test"
 
 	"restic/backend"
 )
 
-// CreateFn is a function that creates a temporary repository for the tests.
-var CreateFn func() (restic.Backend, error)
-
-// OpenFn is a function that opens a previously created temporary repository.
-var OpenFn func() (restic.Backend, error)
-
-// CleanupFn removes temporary files and directories created during the tests.
-var CleanupFn func() error
-
-var but restic.Backend // backendUnderTest
-var butInitialized bool
-
-func open(t testing.TB) restic.Backend {
-	if OpenFn == nil {
-		t.Fatal("OpenFn not set")
-	}
-
-	if CreateFn == nil {
-		t.Fatalf("CreateFn not set")
-	}
-
-	if !butInitialized {
-		be, err := CreateFn()
-		if err != nil {
-			t.Fatalf("Create returned unexpected error: %v", err)
-		}
-
-		but = be
-		butInitialized = true
-	}
-
-	if but == nil {
-		var err error
-		but, err = OpenFn()
-		if err != nil {
-			t.Fatalf("Open returned unexpected error: %v", err)
-		}
-	}
-
-	return but
-}
-
-func close(t testing.TB) {
-	if but == nil {
-		t.Fatalf("trying to close non-existing backend")
-	}
-
-	err := but.Close()
-	if err != nil {
-		t.Fatalf("Close returned unexpected error: %v", err)
-	}
-
-	but = nil
-}
-
-// TestCreate creates a backend.
-func TestCreate(t testing.TB) {
-	if CreateFn == nil {
-		t.Fatalf("CreateFn not set!")
-	}
-
-	be, err := CreateFn()
-	if err != nil {
-		t.Fatalf("Create returned error: %v", err)
-	}
-
-	butInitialized = true
-
-	err = be.Close()
-	if err != nil {
-		t.Fatalf("Close returned error: %v", err)
-	}
-}
-
-// TestOpen opens a previously created backend.
-func TestOpen(t testing.TB) {
-	if OpenFn == nil {
-		t.Fatalf("OpenFn not set!")
-	}
-
-	be, err := OpenFn()
-	if err != nil {
-		t.Fatalf("Open returned error: %v", err)
-	}
-
-	err = be.Close()
-	if err != nil {
-		t.Fatalf("Close returned error: %v", err)
-	}
+func seedRand(t testing.TB) {
+	seed := time.Now().UnixNano()
+	rand.Seed(seed)
+	t.Logf("rand initialized with seed %d", seed)
 }
 
 // TestCreateWithConfig tests that creating a backend in a location which already
 // has a config file fails.
-func TestCreateWithConfig(t testing.TB) {
-	if CreateFn == nil {
-		t.Fatalf("CreateFn not set")
+func (s *Suite) TestCreateWithConfig(t *testing.T) {
+	b := s.open(t)
+	defer s.close(t, b)
+
+	// remove a config if present
+	cfgHandle := restic.Handle{Type: restic.ConfigFile}
+	cfgPresent, err := b.Test(context.TODO(), cfgHandle)
+	if err != nil {
+		t.Fatalf("unable to test for config: %+v", err)
 	}
 
-	b := open(t)
-	defer close(t)
+	if cfgPresent {
+		remove(t, b, cfgHandle)
+	}
 
 	// save a config
 	store(t, b, restic.ConfigFile, []byte("test config"))
 
 	// now create the backend again, this must fail
-	_, err := CreateFn()
+	_, err = s.Create(s.Config)
 	if err == nil {
 		t.Fatalf("expected error not found for creating a backend with an existing config file")
 	}
 
 	// remove config
-	err = b.Remove(restic.Handle{Type: restic.ConfigFile, Name: ""})
+	err = b.Remove(context.TODO(), restic.Handle{Type: restic.ConfigFile, Name: ""})
 	if err != nil {
-		t.Fatalf("unexpected error removing config: %v", err)
+		t.Fatalf("unexpected error removing config: %+v", err)
 	}
 }
 
 // TestLocation tests that a location string is returned.
-func TestLocation(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestLocation(t *testing.T) {
+	b := s.open(t)
+	defer s.close(t, b)
 
 	l := b.Location()
 	if l == "" {
@@ -148,49 +72,57 @@ func TestLocation(t testing.TB) {
 }
 
 // TestConfig saves and loads a config from the backend.
-func TestConfig(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestConfig(t *testing.T) {
+	b := s.open(t)
+	defer s.close(t, b)
 
 	var testString = "Config"
 
 	// create config and read it back
-	_, err := backend.LoadAll(b, restic.Handle{Type: restic.ConfigFile})
+	_, err := backend.LoadAll(context.TODO(), b, restic.Handle{Type: restic.ConfigFile})
 	if err == nil {
 		t.Fatalf("did not get expected error for non-existing config")
 	}
 
-	err = b.Save(restic.Handle{Type: restic.ConfigFile}, strings.NewReader(testString))
+	err = b.Save(context.TODO(), restic.Handle{Type: restic.ConfigFile}, strings.NewReader(testString))
 	if err != nil {
-		t.Fatalf("Save() error: %v", err)
+		t.Fatalf("Save() error: %+v", err)
 	}
 
 	// try accessing the config with different names, should all return the
 	// same config
 	for _, name := range []string{"", "foo", "bar", "0000000000000000000000000000000000000000000000000000000000000000"} {
 		h := restic.Handle{Type: restic.ConfigFile, Name: name}
-		buf, err := backend.LoadAll(b, h)
+		buf, err := backend.LoadAll(context.TODO(), b, h)
 		if err != nil {
-			t.Fatalf("unable to read config with name %q: %v", name, err)
+			t.Fatalf("unable to read config with name %q: %+v", name, err)
 		}
 
 		if string(buf) != testString {
 			t.Fatalf("wrong data returned, want %q, got %q", testString, string(buf))
 		}
 	}
+
+	// remove the config
+	remove(t, b, restic.Handle{Type: restic.ConfigFile})
 }
 
 // TestLoad tests the backend's Load function.
-func TestLoad(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestLoad(t *testing.T) {
+	seedRand(t)
 
-	_, err := b.Load(restic.Handle{}, 0, 0)
+	b := s.open(t)
+	defer s.close(t, b)
+
+	rd, err := b.Load(context.TODO(), restic.Handle{}, 0, 0)
 	if err == nil {
 		t.Fatalf("Load() did not return an error for invalid handle")
 	}
+	if rd != nil {
+		_ = rd.Close()
+	}
 
-	_, err = b.Load(restic.Handle{Type: restic.DataFile, Name: "foobar"}, 0, 0)
+	err = testLoad(b, restic.Handle{Type: restic.DataFile, Name: "foobar"}, 0, 0)
 	if err == nil {
 		t.Fatalf("Load() did not return an error for non-existing blob")
 	}
@@ -201,12 +133,14 @@ func TestLoad(t testing.TB) {
 	id := restic.Hash(data)
 
 	handle := restic.Handle{Type: restic.DataFile, Name: id.String()}
-	err = b.Save(handle, bytes.NewReader(data))
+	err = b.Save(context.TODO(), handle, bytes.NewReader(data))
 	if err != nil {
-		t.Fatalf("Save() error: %v", err)
+		t.Fatalf("Save() error: %+v", err)
 	}
 
-	rd, err := b.Load(handle, 100, -1)
+	t.Logf("saved %d bytes as %v", length, handle)
+
+	rd, err = b.Load(context.TODO(), handle, 100, -1)
 	if err == nil {
 		t.Fatalf("Load() returned no error for negative offset!")
 	}
@@ -215,7 +149,12 @@ func TestLoad(t testing.TB) {
 		t.Fatalf("Load() returned a non-nil reader for negative offset!")
 	}
 
-	for i := 0; i < 50; i++ {
+	loadTests := 50
+	if s.MinimalData {
+		loadTests = 10
+	}
+
+	for i := 0; i < loadTests; i++ {
 		l := rand.Intn(length + 2000)
 		o := rand.Intn(length + 2000)
 
@@ -223,8 +162,8 @@ func TestLoad(t testing.TB) {
 		if o < len(d) {
 			d = d[o:]
 		} else {
-			o = len(d)
-			d = d[:0]
+			t.Logf("offset == length, skipping test")
+			continue
 		}
 
 		getlen := l
@@ -236,45 +175,73 @@ func TestLoad(t testing.TB) {
 			d = d[:l]
 		}
 
-		rd, err := b.Load(handle, getlen, int64(o))
+		rd, err := b.Load(context.TODO(), handle, getlen, int64(o))
 		if err != nil {
-			t.Errorf("Load(%d, %d) returned unexpected error: %v", l, o, err)
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
+			t.Errorf("Load(%d, %d) returned unexpected error: %+v", l, o, err)
 			continue
 		}
 
 		buf, err := ioutil.ReadAll(rd)
 		if err != nil {
-			t.Errorf("Load(%d, %d) ReadAll() returned unexpected error: %v", l, o, err)
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
+			t.Errorf("Load(%d, %d) ReadAll() returned unexpected error: %+v", l, o, err)
+			if err = rd.Close(); err != nil {
+				t.Errorf("Load(%d, %d) rd.Close() returned error: %+v", l, o, err)
+			}
 			continue
 		}
 
-		if l <= len(d) && len(buf) != l {
+		if l == 0 && len(buf) != len(d) {
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
+			t.Errorf("Load(%d, %d) wrong number of bytes read: want %d, got %d", l, o, len(d), len(buf))
+			if err = rd.Close(); err != nil {
+				t.Errorf("Load(%d, %d) rd.Close() returned error: %+v", l, o, err)
+			}
+			continue
+		}
+
+		if l > 0 && l <= len(d) && len(buf) != l {
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
 			t.Errorf("Load(%d, %d) wrong number of bytes read: want %d, got %d", l, o, l, len(buf))
+			if err = rd.Close(); err != nil {
+				t.Errorf("Load(%d, %d) rd.Close() returned error: %+v", l, o, err)
+			}
 			continue
 		}
 
 		if l > len(d) && len(buf) != len(d) {
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
 			t.Errorf("Load(%d, %d) wrong number of bytes read for overlong read: want %d, got %d", l, o, l, len(buf))
+			if err = rd.Close(); err != nil {
+				t.Errorf("Load(%d, %d) rd.Close() returned error: %+v", l, o, err)
+			}
 			continue
 		}
 
 		if !bytes.Equal(buf, d) {
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
 			t.Errorf("Load(%d, %d) returned wrong bytes", l, o)
+			if err = rd.Close(); err != nil {
+				t.Errorf("Load(%d, %d) rd.Close() returned error: %+v", l, o, err)
+			}
 			continue
 		}
 
 		err = rd.Close()
 		if err != nil {
-			t.Errorf("Load(%d, %d) rd.Close() returned unexpected error: %v", l, o, err)
+			t.Logf("Load, l %v, o %v, len(d) %v, getlen %v", l, o, len(d), getlen)
+			t.Errorf("Load(%d, %d) rd.Close() returned unexpected error: %+v", l, o, err)
 			continue
 		}
 	}
 
-	test.OK(t, b.Remove(handle))
+	test.OK(t, b.Remove(context.TODO(), handle))
 }
 
 type errorCloser struct {
 	io.Reader
+	l int
 	t testing.TB
 }
 
@@ -283,13 +250,24 @@ func (ec errorCloser) Close() error {
 	return errors.New("forbidden method close was called")
 }
 
+func (ec errorCloser) Len() int {
+	return ec.l
+}
+
 // TestSave tests saving data in the backend.
-func TestSave(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestSave(t *testing.T) {
+	seedRand(t)
+
+	b := s.open(t)
+	defer s.close(t, b)
 	var id restic.ID
 
-	for i := 0; i < 10; i++ {
+	saveTests := 10
+	if s.MinimalData {
+		saveTests = 2
+	}
+
+	for i := 0; i < saveTests; i++ {
 		length := rand.Intn(1<<23) + 200000
 		data := test.Random(23, length)
 		// use the first 32 byte as the ID
@@ -299,10 +277,10 @@ func TestSave(t testing.TB) {
 			Type: restic.DataFile,
 			Name: fmt.Sprintf("%s-%d", id, i),
 		}
-		err := b.Save(h, bytes.NewReader(data))
+		err := b.Save(context.TODO(), h, bytes.NewReader(data))
 		test.OK(t, err)
 
-		buf, err := backend.LoadAll(b, h)
+		buf, err := backend.LoadAll(context.TODO(), b, h)
 		test.OK(t, err)
 		if len(buf) != len(data) {
 			t.Fatalf("number of bytes does not match, want %v, got %v", len(data), len(buf))
@@ -312,16 +290,16 @@ func TestSave(t testing.TB) {
 			t.Fatalf("data not equal")
 		}
 
-		fi, err := b.Stat(h)
+		fi, err := b.Stat(context.TODO(), h)
 		test.OK(t, err)
 
 		if fi.Size != int64(len(data)) {
 			t.Fatalf("Stat() returned different size, want %q, got %d", len(data), fi.Size)
 		}
 
-		err = b.Remove(h)
+		err = b.Remove(context.TODO(), h)
 		if err != nil {
-			t.Fatalf("error removing item: %v", err)
+			t.Fatalf("error removing item: %+v", err)
 		}
 	}
 
@@ -339,7 +317,7 @@ func TestSave(t testing.TB) {
 		t.Fatal(err)
 	}
 
-	if _, err = tmpfile.Seek(0, 0); err != nil {
+	if _, err = tmpfile.Seek(0, io.SeekStart); err != nil {
 		t.Fatal(err)
 	}
 
@@ -347,7 +325,22 @@ func TestSave(t testing.TB) {
 
 	// wrap the tempfile in an errorCloser, so we can detect if the backend
 	// closes the reader
-	err = b.Save(h, errorCloser{t: t, Reader: tmpfile})
+	err = b.Save(context.TODO(), h, errorCloser{t: t, l: length, Reader: tmpfile})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	err = delayedRemove(t, b, s.WaitForDelayedRemoval, h)
+	if err != nil {
+		t.Fatalf("error removing item: %+v", err)
+	}
+
+	// try again directly with the temp file
+	if _, err = tmpfile.Seek(588, io.SeekStart); err != nil {
+		t.Fatal(err)
+	}
+
+	err = b.Save(context.TODO(), h, tmpfile)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -356,13 +349,13 @@ func TestSave(t testing.TB) {
 		t.Fatal(err)
 	}
 
-	if err = os.Remove(tmpfile.Name()); err != nil {
-		t.Fatal(err)
+	err = b.Remove(context.TODO(), h)
+	if err != nil {
+		t.Fatalf("error removing item: %+v", err)
 	}
 
-	err = b.Remove(h)
-	if err != nil {
-		t.Fatalf("error removing item: %v", err)
+	if err = os.Remove(tmpfile.Name()); err != nil {
+		t.Fatal(err)
 	}
 }
 
@@ -371,7 +364,7 @@ var filenameTests = []struct {
 	data string
 }{
 	{"1dfc6bc0f06cb255889e9ea7860a5753e8eb9665c9a96627971171b444e3113e", "x"},
-	{"foobar", "foobar"},
+	{"f00b4r", "foobar"},
 	{
 		"1dfc6bc0f06cb255889e9ea7860a5753e8eb9665c9a96627971171b444e3113e4bf8f2d9144cc5420a80f04a4880ad6155fc58903a4fb6457c476c43541dcaa6-5",
 		"foobar content of data blob",
@@ -379,21 +372,21 @@ var filenameTests = []struct {
 }
 
 // TestSaveFilenames tests saving data with various file names in the backend.
-func TestSaveFilenames(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestSaveFilenames(t *testing.T) {
+	b := s.open(t)
+	defer s.close(t, b)
 
 	for i, test := range filenameTests {
 		h := restic.Handle{Name: test.name, Type: restic.DataFile}
-		err := b.Save(h, strings.NewReader(test.data))
+		err := b.Save(context.TODO(), h, strings.NewReader(test.data))
 		if err != nil {
-			t.Errorf("test %d failed: Save() returned %v", i, err)
+			t.Errorf("test %d failed: Save() returned %+v", i, err)
 			continue
 		}
 
-		buf, err := backend.LoadAll(b, h)
+		buf, err := backend.LoadAll(context.TODO(), b, h)
 		if err != nil {
-			t.Errorf("test %d failed: Load() returned %v", i, err)
+			t.Errorf("test %d failed: Load() returned %+v", i, err)
 			continue
 		}
 
@@ -401,9 +394,9 @@ func TestSaveFilenames(t testing.TB) {
 			t.Errorf("test %d: returned wrong bytes", i)
 		}
 
-		err = b.Remove(h)
+		err = b.Remove(context.TODO(), h)
 		if err != nil {
-			t.Errorf("test %d failed: Remove() returned %v", i, err)
+			t.Errorf("test %d failed: Remove() returned %+v", i, err)
 			continue
 		}
 	}
@@ -422,15 +415,84 @@ var testStrings = []struct {
 func store(t testing.TB, b restic.Backend, tpe restic.FileType, data []byte) restic.Handle {
 	id := restic.Hash(data)
 	h := restic.Handle{Name: id.String(), Type: tpe}
-	err := b.Save(h, bytes.NewReader(data))
+	err := b.Save(context.TODO(), h, bytes.NewReader(data))
 	test.OK(t, err)
 	return h
 }
 
+// testLoad loads a blob (but discards its contents).
+func testLoad(b restic.Backend, h restic.Handle, length int, offset int64) error {
+	rd, err := b.Load(context.TODO(), h, 0, 0)
+	if err != nil {
+		return err
+	}
+
+	_, err = io.Copy(ioutil.Discard, rd)
+	cerr := rd.Close()
+	if err == nil {
+		err = cerr
+	}
+	return err
+}
+
+func delayedRemove(t testing.TB, be restic.Backend, maxwait time.Duration, handles ...restic.Handle) error {
+	// Some backend (swift, I'm looking at you) may implement delayed
+	// removal of data. Let's wait a bit if this happens.
+
+	for _, h := range handles {
+		err := be.Remove(context.TODO(), h)
+		if err != nil {
+			return err
+		}
+	}
+
+	for _, h := range handles {
+		start := time.Now()
+		attempt := 0
+		var found bool
+		var err error
+		for time.Since(start) <= maxwait {
+			found, err = be.Test(context.TODO(), h)
+			if err != nil {
+				return err
+			}
+
+			if !found {
+				break
+			}
+
+			time.Sleep(2 * time.Second)
+			attempt++
+		}
+
+		if found {
+			t.Fatalf("removed blob %v still present after %v (%d attempts)", h, time.Since(start), attempt)
+		}
+	}
+
+	return nil
+}
+
+func delayedList(t testing.TB, b restic.Backend, tpe restic.FileType, max int, maxwait time.Duration) restic.IDs {
+	list := restic.NewIDSet()
+	start := time.Now()
+	for i := 0; i < max; i++ {
+		for s := range b.List(context.TODO(), tpe) {
+			id := restic.TestParseID(s)
+			list.Insert(id)
+		}
+		if len(list) < max && time.Since(start) < maxwait {
+			time.Sleep(500 * time.Millisecond)
+		}
+	}
+
+	return list.List()
+}
+
 // TestBackend tests all functions of the backend.
-func TestBackend(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestBackend(t *testing.T) {
+	b := s.open(t)
+	defer s.close(t, b)
 
 	for _, tpe := range []restic.FileType{
 		restic.DataFile, restic.KeyFile, restic.LockFile,
@@ -443,20 +505,20 @@ func TestBackend(t testing.TB) {
 
 			// test if blob is already in repository
 			h := restic.Handle{Type: tpe, Name: id.String()}
-			ret, err := b.Test(h)
+			ret, err := b.Test(context.TODO(), h)
 			test.OK(t, err)
 			test.Assert(t, !ret, "blob was found to exist before creating")
 
 			// try to stat a not existing blob
-			_, err = b.Stat(h)
+			_, err = b.Stat(context.TODO(), h)
 			test.Assert(t, err != nil, "blob data could be extracted before creation")
 
 			// try to read not existing blob
-			_, err = b.Load(h, 0, 0)
-			test.Assert(t, err != nil, "blob reader could be obtained before creation")
+			err = testLoad(b, h, 0, 0)
+			test.Assert(t, err != nil, "blob could be read before creation")
 
 			// try to get string out, should fail
-			ret, err = b.Test(h)
+			ret, err = b.Test(context.TODO(), h)
 			test.OK(t, err)
 			test.Assert(t, !ret, "id %q was found (but should not have)", ts.id)
 		}
@@ -467,7 +529,7 @@ func TestBackend(t testing.TB) {
 
 			// test Load()
 			h := restic.Handle{Type: tpe, Name: ts.id}
-			buf, err := backend.LoadAll(b, h)
+			buf, err := backend.LoadAll(context.TODO(), b, h)
 			test.OK(t, err)
 			test.Equals(t, ts.data, string(buf))
 
@@ -477,7 +539,7 @@ func TestBackend(t testing.TB) {
 			length := end - start
 
 			buf2 := make([]byte, length)
-			rd, err := b.Load(h, len(buf2), int64(start))
+			rd, err := b.Load(context.TODO(), h, len(buf2), int64(start))
 			test.OK(t, err)
 			n, err := io.ReadFull(rd, buf2)
 			test.OK(t, err)
@@ -496,21 +558,21 @@ func TestBackend(t testing.TB) {
 		ts := testStrings[0]
 
 		// create blob
-		err := b.Save(restic.Handle{Type: tpe, Name: ts.id}, strings.NewReader(ts.data))
-		test.Assert(t, err != nil, "expected error, got %v", err)
+		h := restic.Handle{Type: tpe, Name: ts.id}
+		err := b.Save(context.TODO(), h, strings.NewReader(ts.data))
+		test.Assert(t, err != nil, "expected error for %v, got %v", h, err)
 
 		// remove and recreate
-		h := restic.Handle{Type: tpe, Name: ts.id}
-		err = b.Remove(h)
+		err = delayedRemove(t, b, s.WaitForDelayedRemoval, h)
 		test.OK(t, err)
 
 		// test that the blob is gone
-		ok, err := b.Test(h)
+		ok, err := b.Test(context.TODO(), h)
 		test.OK(t, err)
-		test.Assert(t, ok == false, "removed blob still present")
+		test.Assert(t, !ok, "removed blob still present")
 
 		// create blob
-		err = b.Save(h, strings.NewReader(ts.data))
+		err = b.Save(context.TODO(), h, strings.NewReader(ts.data))
 		test.OK(t, err)
 
 		// list items
@@ -522,12 +584,7 @@ func TestBackend(t testing.TB) {
 			IDs = append(IDs, id)
 		}
 
-		list := restic.IDs{}
-
-		for s := range b.List(tpe, nil) {
-			list = append(list, restic.TestParseID(s))
-		}
-
+		list := delayedList(t, b, tpe, len(IDs), s.WaitForDelayedRemoval)
 		if len(IDs) != len(list) {
 			t.Fatalf("wrong number of IDs returned: want %d, got %d", len(IDs), len(list))
 		}
@@ -541,55 +598,41 @@ func TestBackend(t testing.TB) {
 
 		// remove content if requested
 		if test.TestCleanupTempDirs {
+			var handles []restic.Handle
 			for _, ts := range testStrings {
 				id, err := restic.ParseID(ts.id)
 				test.OK(t, err)
 
 				h := restic.Handle{Type: tpe, Name: id.String()}
 
-				found, err := b.Test(h)
+				found, err := b.Test(context.TODO(), h)
 				test.OK(t, err)
+				test.Assert(t, found, fmt.Sprintf("id %q not found", id))
 
-				test.OK(t, b.Remove(h))
-
-				found, err = b.Test(h)
-				test.OK(t, err)
-				test.Assert(t, !found, fmt.Sprintf("id %q not found after removal", id))
+				handles = append(handles, h)
 			}
+
+			test.OK(t, delayedRemove(t, b, s.WaitForDelayedRemoval, handles...))
 		}
 	}
 }
 
 // TestDelete tests the Delete function.
-func TestDelete(t testing.TB) {
-	b := open(t)
-	defer close(t)
+func (s *Suite) TestDelete(t *testing.T) {
+	if !test.TestCleanupTempDirs {
+		t.Skipf("not removing backend, TestCleanupTempDirs is false")
+	}
+
+	b := s.open(t)
+	defer s.close(t, b)
 
 	be, ok := b.(restic.Deleter)
 	if !ok {
 		return
 	}
 
-	err := be.Delete()
+	err := be.Delete(context.TODO())
 	if err != nil {
-		t.Fatalf("error deleting backend: %v", err)
-	}
-}
-
-// TestCleanup runs the cleanup function after all tests are run.
-func TestCleanup(t testing.TB) {
-	if CleanupFn == nil {
-		t.Log("CleanupFn function not set")
-		return
-	}
-
-	if !test.TestCleanupTempDirs {
-		t.Logf("not cleaning up backend")
-		return
-	}
-
-	err := CleanupFn()
-	if err != nil {
-		t.Fatalf("Cleanup returned error: %v", err)
+		t.Fatalf("error deleting backend: %+v", err)
 	}
 }
