@@ -1,6 +1,7 @@
 package restic_test
 
 import (
+	"context"
 	"restic"
 	"testing"
 
@@ -13,10 +14,10 @@ func (s saver) SaveJSONUnpacked(t restic.FileType, arg interface{}) (restic.ID,
 	return s(t, arg)
 }
 
-type loader func(restic.FileType, restic.ID, interface{}) error
+type loader func(context.Context, restic.FileType, restic.ID, interface{}) error
 
-func (l loader) LoadJSONUnpacked(t restic.FileType, id restic.ID, arg interface{}) error {
-	return l(t, id, arg)
+func (l loader) LoadJSONUnpacked(ctx context.Context, t restic.FileType, id restic.ID, arg interface{}) error {
+	return l(ctx, t, id, arg)
 }
 
 func TestConfig(t *testing.T) {
@@ -36,7 +37,7 @@ func TestConfig(t *testing.T) {
 
 	_, err = saver(save).SaveJSONUnpacked(restic.ConfigFile, cfg1)
 
-	load := func(tpe restic.FileType, id restic.ID, arg interface{}) error {
+	load := func(ctx context.Context, tpe restic.FileType, id restic.ID, arg interface{}) error {
 		Assert(t, tpe == restic.ConfigFile,
 			"wrong backend type: got %v, wanted %v",
 			tpe, restic.ConfigFile)
@@ -46,7 +47,7 @@ func TestConfig(t *testing.T) {
 		return nil
 	}
 
-	cfg2, err := restic.LoadConfig(loader(load))
+	cfg2, err := restic.LoadConfig(context.TODO(), loader(load))
 	OK(t, err)
 
 	Assert(t, cfg1 == cfg2,
