@@ -18,3 +18,7 @@ var Errorf = errors.Errorf
 // Wrap wraps an error retrieved from outside of restic. Wrapped so that this
 // package does not appear in the stack trace.
 var Wrap = errors.Wrap
+
+// Wrapf returns an error annotating err with the format specifier. If err is
+// nil, Wrapf returns nil.
+var Wrapf = errors.Wrapf
