@@ -21,32 +21,26 @@ const blockSize = 512
 var _ = fs.HandleReader(&file{})
 var _ = fs.HandleReleaser(&file{})
 
-// BlobLoader is an abstracted repository with a reduced set of methods used
-// for fuse operations.
-type BlobLoader interface {
-	LookupBlobSize(restic.ID, restic.BlobType) (uint, error)
-	LoadBlob(restic.BlobType, restic.ID, []byte) (int, error)
-}
-
 type file struct {
-	repo        BlobLoader
-	node        *restic.Node
-	ownerIsRoot bool
+	root  *Root
+	node  *restic.Node
+	inode uint64
 
 	sizes []int
 	blobs [][]byte
 }
 
-const defaultBlobSize = 128 * 1024
-
-func newFile(repo BlobLoader, node *restic.Node, ownerIsRoot bool) (*file, error) {
+func newFile(ctx context.Context, root *Root, inode uint64, node *restic.Node) (fusefile *file, err error) {
 	debug.Log("create new file for %v with %d blobs", node.Name, len(node.Content))
 	var bytes uint64
 	sizes := make([]int, len(node.Content))
 	for i, id := range node.Content {
-		size, err := repo.LookupBlobSize(id, restic.DataBlob)
-		if err != nil {
-			return nil, err
+		size, ok := root.blobSizeCache.Lookup(id)
+		if !ok {
+			size, err = root.repo.LookupBlobSize(id, restic.DataBlob)
+			if err != nil {
+				return nil, err
+			}
 		}
 
 		sizes[i] = int(size)
@@ -59,33 +53,36 @@ func newFile(repo BlobLoader, node *restic.Node, ownerIsRoot bool) (*file, error
 	}
 
 	return &file{
-		repo:        repo,
-		node:        node,
-		sizes:       sizes,
-		blobs:       make([][]byte, len(node.Content)),
-		ownerIsRoot: ownerIsRoot,
+		inode: inode,
+		root:  root,
+		node:  node,
+		sizes: sizes,
+		blobs: make([][]byte, len(node.Content)),
 	}, nil
 }
 
 func (f *file) Attr(ctx context.Context, a *fuse.Attr) error {
 	debug.Log("Attr(%v)", f.node.Name)
-	a.Inode = f.node.Inode
+	a.Inode = f.inode
 	a.Mode = f.node.Mode
 	a.Size = f.node.Size
 	a.Blocks = (f.node.Size / blockSize) + 1
 	a.BlockSize = blockSize
+	a.Nlink = uint32(f.node.Links)
 
-	if !f.ownerIsRoot {
+	if !f.root.cfg.OwnerIsRoot {
 		a.Uid = f.node.UID
 		a.Gid = f.node.GID
 	}
 	a.Atime = f.node.AccessTime
 	a.Ctime = f.node.ChangeTime
 	a.Mtime = f.node.ModTime
+
 	return nil
+
 }
 
-func (f *file) getBlobAt(i int) (blob []byte, err error) {
+func (f *file) getBlobAt(ctx context.Context, i int) (blob []byte, err error) {
 	debug.Log("getBlobAt(%v, %v)", f.node.Name, i)
 	if f.blobs[i] != nil {
 		return f.blobs[i], nil
@@ -97,7 +94,7 @@ func (f *file) getBlobAt(i int) (blob []byte, err error) {
 	}
 
 	buf := restic.NewBlobBuffer(f.sizes[i])
-	n, err := f.repo.LoadBlob(restic.DataBlob, f.node.Content[i], buf)
+	n, err := f.root.repo.LoadBlob(ctx, restic.DataBlob, f.node.Content[i], buf)
 	if err != nil {
 		debug.Log("LoadBlob(%v, %v) failed: %v", f.node.Name, f.node.Content[i], err)
 		return nil, err
@@ -134,13 +131,13 @@ func (f *file) Read(ctx context.Context, req *fuse.ReadRequest, resp *fuse.ReadR
 	readBytes := 0
 	remainingBytes := req.Size
 	for i := startContent; remainingBytes > 0 && i < len(f.sizes); i++ {
-		blob, err := f.getBlobAt(i)
+		blob, err := f.getBlobAt(ctx, i)
 		if err != nil {
 			return err
 		}
 
 		if offset > 0 {
-			blob = blob[offset:len(blob)]
+			blob = blob[offset:]
 			offset = 0
 		}
 
@@ -161,3 +158,21 @@ func (f *file) Release(ctx context.Context, req *fuse.ReleaseRequest) error {
 	}
 	return nil
 }
+
+func (f *file) Listxattr(ctx context.Context, req *fuse.ListxattrRequest, resp *fuse.ListxattrResponse) error {
+	debug.Log("Listxattr(%v, %v)", f.node.Name, req.Size)
+	for _, attr := range f.node.ExtendedAttributes {
+		resp.Append(attr.Name)
+	}
+	return nil
+}
+
+func (f *file) Getxattr(ctx context.Context, req *fuse.GetxattrRequest, resp *fuse.GetxattrResponse) error {
+	debug.Log("Getxattr(%v, %v, %v)", f.node.Name, req.Name, req.Size)
+	attrval := f.node.GetExtendedAttribute(req.Name)
+	if attrval != nil {
+		resp.Xattr = attrval
+		return nil
+	}
+	return fuse.ErrNoXattr
+}
