@@ -14,6 +14,7 @@ import (
 	"restic/repository"
 
 	"bazil.org/fuse"
+	"bazil.org/fuse/fs"
 
 	"restic"
 	. "restic/test"
@@ -34,9 +35,7 @@ func testRead(t testing.TB, f *file, offset, length int, data []byte) {
 }
 
 func firstSnapshotID(t testing.TB, repo restic.Repository) (first restic.ID) {
-	done := make(chan struct{})
-	defer close(done)
-	for id := range repo.List(restic.SnapshotFile, done) {
+	for id := range repo.List(context.TODO(), restic.SnapshotFile) {
 		if first.IsNull() {
 			first = id
 		}
@@ -46,13 +45,13 @@ func firstSnapshotID(t testing.TB, repo restic.Repository) (first restic.ID) {
 
 func loadFirstSnapshot(t testing.TB, repo restic.Repository) *restic.Snapshot {
 	id := firstSnapshotID(t, repo)
-	sn, err := restic.LoadSnapshot(repo, id)
+	sn, err := restic.LoadSnapshot(context.TODO(), repo, id)
 	OK(t, err)
 	return sn
 }
 
 func loadTree(t testing.TB, repo restic.Repository, id restic.ID) *restic.Tree {
-	tree, err := repo.LoadTree(id)
+	tree, err := repo.LoadTree(context.TODO(), id)
 	OK(t, err)
 	return tree
 }
@@ -87,7 +86,7 @@ func TestFuseFile(t *testing.T) {
 		filesize += uint64(size)
 
 		buf := restic.NewBlobBuffer(int(size))
-		n, err := repo.LoadBlob(restic.DataBlob, id, buf)
+		n, err := repo.LoadBlob(context.TODO(), restic.DataBlob, id, buf)
 		OK(t, err)
 
 		if uint(n) != size {
@@ -110,13 +109,21 @@ func TestFuseFile(t *testing.T) {
 		Size:    filesize,
 		Content: content,
 	}
-	f, err := newFile(repo, node, false)
+	root := &Root{
+		blobSizeCache: NewBlobSizeCache(context.TODO(), repo.Index()),
+		repo:          repo,
+	}
+
+	t.Logf("blob cache has %d entries", len(root.blobSizeCache.m))
+
+	inode := fs.GenerateDynamicInode(1, "foo")
+	f, err := newFile(context.TODO(), root, inode, node)
 	OK(t, err)
 
 	attr := fuse.Attr{}
 	OK(t, f.Attr(ctx, &attr))
 
-	Equals(t, node.Inode, attr.Inode)
+	Equals(t, inode, attr.Inode)
 	Equals(t, node.Mode, attr.Mode)
 	Equals(t, node.Size, attr.Size)
 	Equals(t, (node.Size/uint64(attr.BlockSize))+1, attr.Blocks)
