@@ -2,11 +2,13 @@
 package index
 
 import (
+	"context"
 	"fmt"
 	"os"
 	"restic"
 	"restic/debug"
 	"restic/list"
+	"restic/pack"
 	"restic/worker"
 
 	"restic/errors"
@@ -32,25 +34,29 @@ func newIndex() *Index {
 	}
 }
 
-// New creates a new index for repo from scratch.
-func New(repo restic.Repository, p *restic.Progress) (*Index, error) {
-	done := make(chan struct{})
-	defer close(done)
-
+// New creates a new index for repo from scratch. InvalidFiles contains all IDs
+// of files  that cannot be listed successfully.
+func New(ctx context.Context, repo restic.Repository, ignorePacks restic.IDSet, p *restic.Progress) (idx *Index, invalidFiles restic.IDs, err error) {
 	p.Start()
 	defer p.Done()
 
 	ch := make(chan worker.Job)
-	go list.AllPacks(repo, ch, done)
+	go list.AllPacks(ctx, repo, ignorePacks, ch)
 
-	idx := newIndex()
+	idx = newIndex()
 
 	for job := range ch {
 		p.Report(restic.Stat{Blobs: 1})
 
 		packID := job.Data.(restic.ID)
 		if job.Error != nil {
-			fmt.Fprintf(os.Stderr, "unable to list pack %v: %v\n", packID.Str(), job.Error)
+			cause := errors.Cause(job.Error)
+			if _, ok := cause.(pack.InvalidFileError); ok {
+				invalidFiles = append(invalidFiles, packID)
+				continue
+			}
+
+			fmt.Fprintf(os.Stderr, "pack file cannot be listed %v: %v\n", packID.Str(), job.Error)
 			continue
 		}
 
@@ -60,11 +66,11 @@ func New(repo restic.Repository, p *restic.Progress) (*Index, error) {
 
 		err := idx.AddPack(packID, j.Size(), j.Entries())
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 	}
 
-	return idx, nil
+	return idx, invalidFiles, nil
 }
 
 type packJSON struct {
@@ -84,11 +90,11 @@ type indexJSON struct {
 	Packs      []*packJSON `json:"packs"`
 }
 
-func loadIndexJSON(repo restic.Repository, id restic.ID) (*indexJSON, error) {
+func loadIndexJSON(ctx context.Context, repo restic.Repository, id restic.ID) (*indexJSON, error) {
 	debug.Log("process index %v\n", id.Str())
 
 	var idx indexJSON
-	err := repo.LoadJSONUnpacked(restic.IndexFile, id, &idx)
+	err := repo.LoadJSONUnpacked(ctx, restic.IndexFile, id, &idx)
 	if err != nil {
 		return nil, err
 	}
@@ -97,25 +103,22 @@ func loadIndexJSON(repo restic.Repository, id restic.ID) (*indexJSON, error) {
 }
 
 // Load creates an index by loading all index files from the repo.
-func Load(repo restic.Repository, p *restic.Progress) (*Index, error) {
+func Load(ctx context.Context, repo restic.Repository, p *restic.Progress) (*Index, error) {
 	debug.Log("loading indexes")
 
 	p.Start()
 	defer p.Done()
 
-	done := make(chan struct{})
-	defer close(done)
-
 	supersedes := make(map[restic.ID]restic.IDSet)
 	results := make(map[restic.ID]map[restic.ID]Pack)
 
 	index := newIndex()
 
-	for id := range repo.List(restic.IndexFile, done) {
+	for id := range repo.List(ctx, restic.IndexFile) {
 		p.Report(restic.Stat{Blobs: 1})
 
 		debug.Log("Load index %v", id.Str())
-		idx, err := loadIndexJSON(repo, id)
+		idx, err := loadIndexJSON(ctx, repo, id)
 		if err != nil {
 			return nil, err
 		}
@@ -250,17 +253,17 @@ func (idx *Index) FindBlob(h restic.BlobHandle) (result []Location, err error) {
 }
 
 // Save writes the complete index to the repo.
-func (idx *Index) Save(repo restic.Repository, supersedes restic.IDs) (restic.ID, error) {
+func (idx *Index) Save(ctx context.Context, repo restic.Repository, supersedes restic.IDs) (restic.ID, error) {
 	packs := make(map[restic.ID][]restic.Blob, len(idx.Packs))
 	for id, p := range idx.Packs {
 		packs[id] = p.Entries
 	}
 
-	return Save(repo, packs, supersedes)
+	return Save(ctx, repo, packs, supersedes)
 }
 
 // Save writes a new index containing the given packs.
-func Save(repo restic.Repository, packs map[restic.ID][]restic.Blob, supersedes restic.IDs) (restic.ID, error) {
+func Save(ctx context.Context, repo restic.Repository, packs map[restic.ID][]restic.Blob, supersedes restic.IDs) (restic.ID, error) {
 	idx := &indexJSON{
 		Supersedes: supersedes,
 		Packs:      make([]*packJSON, 0, len(packs)),
@@ -285,5 +288,5 @@ func Save(repo restic.Repository, packs map[restic.ID][]restic.Blob, supersedes
 		idx.Packs = append(idx.Packs, p)
 	}
 
-	return repo.SaveJSONUnpacked(restic.IndexFile, idx)
+	return repo.SaveJSONUnpacked(ctx, restic.IndexFile, idx)
 }
