@@ -1,6 +1,7 @@
 package restic
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"os"
@@ -12,31 +13,39 @@ import (
 
 	"restic/errors"
 
-	"runtime"
-
+	"bytes"
 	"restic/debug"
 	"restic/fs"
+	"runtime"
 )
 
+// ExtendedAttribute is a tuple storing the xattr name and value.
+type ExtendedAttribute struct {
+	Name  string `json:"name"`
+	Value []byte `json:"value"`
+}
+
 // Node is a file, directory or other item in a backup.
 type Node struct {
-	Name       string      `json:"name"`
-	Type       string      `json:"type"`
-	Mode       os.FileMode `json:"mode,omitempty"`
-	ModTime    time.Time   `json:"mtime,omitempty"`
-	AccessTime time.Time   `json:"atime,omitempty"`
-	ChangeTime time.Time   `json:"ctime,omitempty"`
-	UID        uint32      `json:"uid"`
-	GID        uint32      `json:"gid"`
-	User       string      `json:"user,omitempty"`
-	Group      string      `json:"group,omitempty"`
-	Inode      uint64      `json:"inode,omitempty"`
-	Size       uint64      `json:"size,omitempty"`
-	Links      uint64      `json:"links,omitempty"`
-	LinkTarget string      `json:"linktarget,omitempty"`
-	Device     uint64      `json:"device,omitempty"`
-	Content    IDs         `json:"content"`
-	Subtree    *ID         `json:"subtree,omitempty"`
+	Name               string              `json:"name"`
+	Type               string              `json:"type"`
+	Mode               os.FileMode         `json:"mode,omitempty"`
+	ModTime            time.Time           `json:"mtime,omitempty"`
+	AccessTime         time.Time           `json:"atime,omitempty"`
+	ChangeTime         time.Time           `json:"ctime,omitempty"`
+	UID                uint32              `json:"uid"`
+	GID                uint32              `json:"gid"`
+	User               string              `json:"user,omitempty"`
+	Group              string              `json:"group,omitempty"`
+	Inode              uint64              `json:"inode,omitempty"`
+	DeviceID           uint64              `json:"device_id,omitempty"` // device id of the file, stat.st_dev
+	Size               uint64              `json:"size,omitempty"`
+	Links              uint64              `json:"links,omitempty"`
+	LinkTarget         string              `json:"linktarget,omitempty"`
+	ExtendedAttributes []ExtendedAttribute `json:"extended_attributes,omitempty"`
+	Device             uint64              `json:"device,omitempty"` // in case of Type == "dev", stat.st_rdev
+	Content            IDs                 `json:"content"`
+	Subtree            *ID                 `json:"subtree,omitempty"`
 
 	Error string `json:"error,omitempty"`
 
@@ -56,7 +65,8 @@ func (node Node) String() string {
 	return fmt.Sprintf("<Node(%s) %s>", node.Type, node.Name)
 }
 
-// NodeFromFileInfo returns a new node from the given path and FileInfo.
+// NodeFromFileInfo returns a new node from the given path and FileInfo. It
+// returns the first error that is encountered, together with a node.
 func NodeFromFileInfo(path string, fi os.FileInfo) (*Node, error) {
 	mask := os.ModePerm | os.ModeType | os.ModeSetuid | os.ModeSetgid | os.ModeSticky
 	node := &Node{
@@ -96,8 +106,18 @@ func nodeTypeFromFileInfo(fi os.FileInfo) string {
 	return ""
 }
 
+// GetExtendedAttribute gets the extended attribute.
+func (node Node) GetExtendedAttribute(a string) []byte {
+	for _, attr := range node.ExtendedAttributes {
+		if attr.Name == a {
+			return attr.Value
+		}
+	}
+	return nil
+}
+
 // CreateAt creates the node at the given path and restores all the meta data.
-func (node *Node) CreateAt(path string, repo Repository) error {
+func (node *Node) CreateAt(ctx context.Context, path string, repo Repository, idx *HardlinkIndex) error {
 	debug.Log("create node %v at %v", node.Name, path)
 
 	switch node.Type {
@@ -106,7 +126,7 @@ func (node *Node) CreateAt(path string, repo Repository) error {
 			return err
 		}
 	case "file":
-		if err := node.createFileAt(path, repo); err != nil {
+		if err := node.createFileAt(ctx, path, repo, idx); err != nil {
 			return err
 		}
 	case "symlink":
@@ -140,28 +160,46 @@ func (node *Node) CreateAt(path string, repo Repository) error {
 }
 
 func (node Node) restoreMetadata(path string) error {
-	var err error
+	var firsterr error
 
-	err = lchown(path, int(node.UID), int(node.GID))
-	if err != nil {
-		return errors.Wrap(err, "Lchown")
+	if err := lchown(path, int(node.UID), int(node.GID)); err != nil {
+		firsterr = errors.Wrap(err, "Lchown")
 	}
 
 	if node.Type != "symlink" {
-		err = fs.Chmod(path, node.Mode)
-		if err != nil {
-			return errors.Wrap(err, "Chmod")
+		if err := fs.Chmod(path, node.Mode); err != nil {
+			if firsterr != nil {
+				firsterr = errors.Wrap(err, "Chmod")
+			}
 		}
 	}
 
 	if node.Type != "dir" {
-		err = node.RestoreTimestamps(path)
-		if err != nil {
+		if err := node.RestoreTimestamps(path); err != nil {
 			debug.Log("error restoring timestamps for dir %v: %v", path, err)
-			return err
+			if firsterr != nil {
+				firsterr = err
+			}
 		}
 	}
 
+	if err := node.restoreExtendedAttributes(path); err != nil {
+		debug.Log("error restoring extended attributes for %v: %v", path, err)
+		if firsterr != nil {
+			firsterr = err
+		}
+	}
+
+	return firsterr
+}
+
+func (node Node) restoreExtendedAttributes(path string) error {
+	for _, attr := range node.ExtendedAttributes {
+		err := Setxattr(path, attr.Name, attr.Value)
+		if err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
@@ -191,8 +229,19 @@ func (node Node) createDirAt(path string) error {
 	return nil
 }
 
-func (node Node) createFileAt(path string, repo Repository) error {
-	f, err := fs.OpenFile(path, os.O_CREATE|os.O_WRONLY, 0600)
+func (node Node) createFileAt(ctx context.Context, path string, repo Repository, idx *HardlinkIndex) error {
+	if node.Links > 1 && idx.Has(node.Inode, node.DeviceID) {
+		if err := fs.Remove(path); !os.IsNotExist(err) {
+			return errors.Wrap(err, "RemoveCreateHardlink")
+		}
+		err := fs.Link(idx.GetFilename(node.Inode, node.DeviceID), path)
+		if err != nil {
+			return errors.Wrap(err, "CreateHardlink")
+		}
+		return nil
+	}
+
+	f, err := fs.OpenFile(path, os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0600)
 	defer f.Close()
 
 	if err != nil {
@@ -207,11 +256,11 @@ func (node Node) createFileAt(path string, repo Repository) error {
 		}
 
 		buf = buf[:cap(buf)]
-		if uint(len(buf)) < size {
+		if len(buf) < CiphertextLength(int(size)) {
 			buf = NewBlobBuffer(int(size))
 		}
 
-		n, err := repo.LoadBlob(DataBlob, id, buf)
+		n, err := repo.LoadBlob(ctx, DataBlob, id, buf)
 		if err != nil {
 			return err
 		}
@@ -223,6 +272,10 @@ func (node Node) createFileAt(path string, repo Repository) error {
 		}
 	}
 
+	if node.Links > 1 {
+		idx.Add(node.Inode, node.DeviceID, path)
+	}
+
 	return nil
 }
 
@@ -325,6 +378,9 @@ func (node Node) Equals(other Node) bool {
 	if node.Inode != other.Inode {
 		return false
 	}
+	if node.DeviceID != other.DeviceID {
+		return false
+	}
 	if node.Size != other.Size {
 		return false
 	}
@@ -340,6 +396,9 @@ func (node Node) Equals(other Node) bool {
 	if !node.sameContent(other) {
 		return false
 	}
+	if !node.sameExtendedAttributes(other) {
+		return false
+	}
 	if node.Subtree != nil {
 		if other.Subtree == nil {
 			return false
@@ -378,6 +437,51 @@ func (node Node) sameContent(other Node) bool {
 			return false
 		}
 	}
+	return true
+}
+
+func (node Node) sameExtendedAttributes(other Node) bool {
+	if len(node.ExtendedAttributes) != len(other.ExtendedAttributes) {
+		return false
+	}
+
+	// build a set of all attributes that node has
+	type mapvalue struct {
+		value   []byte
+		present bool
+	}
+	attributes := make(map[string]mapvalue)
+	for _, attr := range node.ExtendedAttributes {
+		attributes[attr.Name] = mapvalue{value: attr.Value}
+	}
+
+	for _, attr := range other.ExtendedAttributes {
+		v, ok := attributes[attr.Name]
+		if !ok {
+			// extended attribute is not set for node
+			debug.Log("other node has attribute %v, which is not present in node", attr.Name)
+			return false
+
+		}
+
+		if !bytes.Equal(v.value, attr.Value) {
+			// attribute has different value
+			debug.Log("attribute %v has different value", attr.Name)
+			return false
+		}
+
+		// remember that this attribute is present in other.
+		v.present = true
+		attributes[attr.Name] = v
+	}
+
+	// check for attributes that are not present in other
+	for name, v := range attributes {
+		if !v.present {
+			debug.Log("attribute %v not present in other node", name)
+			return false
+		}
+	}
 
 	return true
 }
@@ -469,6 +573,7 @@ func (node *Node) fillExtra(path string, fi os.FileInfo) error {
 	}
 
 	node.Inode = uint64(stat.ino())
+	node.DeviceID = uint64(stat.dev())
 
 	node.fillTimes(stat)
 
@@ -485,18 +590,56 @@ func (node *Node) fillExtra(path string, fi os.FileInfo) error {
 	case "dir":
 	case "symlink":
 		node.LinkTarget, err = fs.Readlink(path)
-		err = errors.Wrap(err, "Readlink")
+		node.Links = uint64(stat.nlink())
+		if err != nil {
+			return errors.Wrap(err, "Readlink")
+		}
 	case "dev":
 		node.Device = uint64(stat.rdev())
+		node.Links = uint64(stat.nlink())
 	case "chardev":
 		node.Device = uint64(stat.rdev())
+		node.Links = uint64(stat.nlink())
 	case "fifo":
 	case "socket":
 	default:
-		err = errors.Errorf("invalid node type %q", node.Type)
+		return errors.Errorf("invalid node type %q", node.Type)
 	}
 
-	return err
+	if err = node.fillExtendedAttributes(path); err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func (node *Node) fillExtendedAttributes(path string) error {
+	if node.Type == "symlink" {
+		return nil
+	}
+
+	xattrs, err := Listxattr(path)
+	debug.Log("fillExtendedAttributes(%v) %v %v", path, xattrs, err)
+	if err != nil {
+		return err
+	}
+
+	node.ExtendedAttributes = make([]ExtendedAttribute, 0, len(xattrs))
+	for _, attr := range xattrs {
+		attrVal, err := Getxattr(path, attr)
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "can not obtain extended attribute %v for %v:\n", attr, path)
+			continue
+		}
+		attr := ExtendedAttribute{
+			Name:  attr,
+			Value: attrVal,
+		}
+
+		node.ExtendedAttributes = append(node.ExtendedAttributes, attr)
+	}
+
+	return nil
 }
 
 type statT interface {
