@@ -1,6 +1,7 @@
 package restic
 
 import (
+	"context"
 	"io"
 	"restic/debug"
 )
@@ -11,7 +12,7 @@ type backendReaderAt struct {
 }
 
 func (brd backendReaderAt) ReadAt(p []byte, offset int64) (n int, err error) {
-	return ReadAt(brd.be, brd.h, offset, p)
+	return ReadAt(context.TODO(), brd.be, brd.h, offset, p)
 }
 
 // ReaderAt returns an io.ReaderAt for a file in the backend.
@@ -20,9 +21,9 @@ func ReaderAt(be Backend, h Handle) io.ReaderAt {
 }
 
 // ReadAt reads from the backend handle h at the given position.
-func ReadAt(be Backend, h Handle, offset int64, p []byte) (n int, err error) {
+func ReadAt(ctx context.Context, be Backend, h Handle, offset int64, p []byte) (n int, err error) {
 	debug.Log("ReadAt(%v) at %v, len %v", h, offset, len(p))
-	rd, err := be.Load(h, len(p), offset)
+	rd, err := be.Load(ctx, h, len(p), offset)
 	if err != nil {
 		return 0, err
 	}
