@@ -1,6 +1,7 @@
 package repository
 
 import (
+	"context"
 	"encoding/json"
 	"io"
 	"restic"
@@ -205,10 +206,10 @@ func (idx *Index) AddToSupersedes(ids ...restic.ID) error {
 	return nil
 }
 
-// Each returns a channel that yields all blobs known to the index. If done is
-// closed, the background goroutine terminates. This blocks any modification of
-// the index.
-func (idx *Index) Each(done chan struct{}) <-chan restic.PackedBlob {
+// Each returns a channel that yields all blobs known to the index. When the
+// context is cancelled, the background goroutine terminates. This blocks any
+// modification of the index.
+func (idx *Index) Each(ctx context.Context) <-chan restic.PackedBlob {
 	idx.m.Lock()
 
 	ch := make(chan restic.PackedBlob)
@@ -222,7 +223,7 @@ func (idx *Index) Each(done chan struct{}) <-chan restic.PackedBlob {
 		for h, packs := range idx.pack {
 			for _, blob := range packs {
 				select {
-				case <-done:
+				case <-ctx.Done():
 					return
 				case ch <- restic.PackedBlob{
 					Blob: restic.Blob{
@@ -392,7 +393,7 @@ func (idx *Index) SetID(id restic.ID) error {
 	defer idx.m.Unlock()
 
 	if !idx.final {
-		return errors.New("indexs is not final")
+		return errors.New("index is not final")
 	}
 
 	if !idx.id.IsNull() {
@@ -519,10 +520,10 @@ func DecodeOldIndex(buf []byte) (idx *Index, err error) {
 }
 
 // LoadIndexWithDecoder loads the index and decodes it with fn.
-func LoadIndexWithDecoder(repo restic.Repository, id restic.ID, fn func([]byte) (*Index, error)) (idx *Index, err error) {
+func LoadIndexWithDecoder(ctx context.Context, repo restic.Repository, id restic.ID, fn func([]byte) (*Index, error)) (idx *Index, err error) {
 	debug.Log("Loading index %v", id.Str())
 
-	buf, err := repo.LoadAndDecrypt(restic.IndexFile, id)
+	buf, err := repo.LoadAndDecrypt(ctx, restic.IndexFile, id)
 	if err != nil {
 		return nil, err
 	}
