@@ -1,9 +1,9 @@
 package repository
 
 import (
+	"context"
 	"crypto/sha256"
 	"io"
-	"io/ioutil"
 	"os"
 	"restic"
 	"sync"
@@ -19,7 +19,7 @@ import (
 
 // Saver implements saving data in a backend.
 type Saver interface {
-	Save(restic.Handle, io.Reader) error
+	Save(context.Context, restic.Handle, io.Reader) error
 }
 
 // Packer holds a pack.Packer together with a hash writer.
@@ -78,9 +78,9 @@ func (r *packerManager) findPacker(size uint) (packer *Packer, err error) {
 
 	// no suitable packer found, return new
 	debug.Log("create new pack for %d bytes", size)
-	tmpfile, err := ioutil.TempFile("", "restic-temp-pack-")
+	tmpfile, err := fs.TempFile("", "restic-temp-pack-")
 	if err != nil {
-		return nil, errors.Wrap(err, "ioutil.TempFile")
+		return nil, errors.Wrap(err, "fs.TempFile")
 	}
 
 	hw := hashing.NewWriter(tmpfile, sha256.New())
@@ -105,7 +105,7 @@ func (r *packerManager) insertPacker(p *Packer) {
 
 // savePacker stores p in the backend.
 func (r *Repository) savePacker(p *Packer) error {
-	debug.Log("save packer with %d blobs\n", p.Packer.Count())
+	debug.Log("save packer with %d blobs (%d bytes)\n", p.Packer.Count(), p.Packer.Size())
 	_, err := p.Packer.Finalize()
 	if err != nil {
 		return err
@@ -119,7 +119,7 @@ func (r *Repository) savePacker(p *Packer) error {
 	id := restic.IDFromHash(p.hw.Sum(nil))
 	h := restic.Handle{Type: restic.DataFile, Name: id.String()}
 
-	err = r.be.Save(h, p.tmpfile)
+	err = r.be.Save(context.TODO(), h, p.tmpfile)
 	if err != nil {
 		debug.Log("Save(%v) error: %v", h, err)
 		return err
@@ -132,7 +132,7 @@ func (r *Repository) savePacker(p *Packer) error {
 		return errors.Wrap(err, "close tempfile")
 	}
 
-	err = fs.Remove(p.tmpfile.Name())
+	err = fs.RemoveIfExists(p.tmpfile.Name())
 	if err != nil {
 		return errors.Wrap(err, "Remove")
 	}
