@@ -1,12 +1,14 @@
 package repository
 
 import (
+	"context"
 	"io"
 	"math/rand"
 	"os"
 	"restic"
 	"restic/backend/mem"
 	"restic/crypto"
+	"restic/fs"
 	"restic/mock"
 	"testing"
 )
@@ -51,7 +53,7 @@ func saveFile(t testing.TB, be Saver, f *os.File, id restic.ID) {
 	h := restic.Handle{Type: restic.DataFile, Name: id.String()}
 	t.Logf("save file %v", h)
 
-	if err := be.Save(h, f); err != nil {
+	if err := be.Save(context.TODO(), h, f); err != nil {
 		t.Fatal(err)
 	}
 
@@ -59,7 +61,7 @@ func saveFile(t testing.TB, be Saver, f *os.File, id restic.ID) {
 		t.Fatal(err)
 	}
 
-	if err := os.Remove(f.Name()); err != nil {
+	if err := fs.RemoveIfExists(f.Name()); err != nil {
 		t.Fatal(err)
 	}
 }
@@ -144,7 +146,7 @@ func BenchmarkPackerManager(t *testing.B) {
 	rnd := newRandReader(rand.NewSource(23))
 
 	be := &mock.Backend{
-		SaveFn: func(restic.Handle, io.Reader) error { return nil },
+		SaveFn: func(context.Context, restic.Handle, io.Reader) error { return nil },
 	}
 	blobBuf := make([]byte, maxBlobSize)
 
